@@ -30,6 +30,8 @@ module Data.Store.Core
     , pokeFromForeignPtr, peekToPlainForeignPtr, pokeFromPtr
       -- * ByteArray
     , pokeFromByteArray, peekToByteArray
+      -- * Creation of PokeState / PeekState
+    , unsafeMakePokeState, unsafeMakePeekState, maybeAlignmentBufferSize
     ) where
 
 import           Control.Applicative
@@ -40,14 +42,14 @@ import           Control.Monad.Primitive (PrimMonad (..))
 import           Data.ByteString (ByteString)
 import qualified Data.ByteString.Internal as BS
 import           Data.Monoid ((<>))
-import           Data.Primitive.ByteArray
+import           Data.Primitive.ByteArray (ByteArray, MutableByteArray(..), newByteArray, unsafeFreezeByteArray)
 import qualified Data.Text as T
 import           Data.Typeable
 import           Data.Word
 import           Foreign.ForeignPtr (ForeignPtr, withForeignPtr, castForeignPtr)
 import           Foreign.Ptr
 import           Foreign.Storable as Storable
-import           GHC.Prim (unsafeCoerce#, RealWorld, copyByteArrayToAddr#, copyAddrToByteArray#)
+import           GHC.Prim (unsafeCoerce#, RealWorld, ByteArray#, copyByteArrayToAddr#, copyAddrToByteArray#)
 import           GHC.Ptr (Ptr(..))
 import           GHC.Types (IO(..), Int(..))
 import           Prelude
@@ -138,6 +140,23 @@ newtype PokeState = PokeState
     }
 #endif
 
+-- | Make a 'PokeState' from a buffer pointer.
+--
+-- The first argument is a pointer to the memory to write to. The second
+-- argument is an IO action which is invoked if the store-core package
+-- was built with the @force-alignment@ flag. The action should yield a
+-- pointer to scratch memory as large as 'maybeAlignmentBufferSize'.
+--
+-- Since 0.4.2
+unsafeMakePokeState :: Ptr Word8 -- ^ pokeStatePtr
+                    -> IO (Ptr Word8) -- ^ action to produce pokeStateAlignPtr
+                    -> IO PokeState
+#if ALIGNED_MEMORY
+unsafeMakePokeState ptr f = PokeState ptr <$> f
+#else
+unsafeMakePokeState ptr _ = return $ PokeState ptr
+#endif
+
 -- | Exception thrown while running 'poke'. Note that other types of
 -- exceptions could also be thrown. Invocations of 'fail' in the 'Poke'
 -- monad causes this exception to be thrown.
@@ -243,6 +262,23 @@ newtype PeekState = PeekState
     { peekStateEndPtr :: Ptr Word8 }
 #endif
 
+-- | Make a 'PeekState' from a buffer pointer.
+--
+-- The first argument is a pointer to the memory to write to. The second
+-- argument is an IO action which is invoked if the store-core package
+-- was built with the @force-alignment@ flag. The action should yield a
+-- pointer to scratch memory as large as 'maybeAlignmentBufferSize'.
+--
+-- Since 0.4.2
+unsafeMakePeekState :: Ptr Word8 -- ^ peekStateEndPtr
+                    -> IO (Ptr Word8) -- ^ action to produce peekStateAlignPtr
+                    -> IO PeekState
+#if ALIGNED_MEMORY
+unsafeMakePeekState ptr f = PeekState ptr <$> f
+#else
+unsafeMakePeekState ptr _ = return $ PeekState ptr
+#endif
+
 -- | Exception thrown while running 'peek'. Note that other types of
 -- exceptions can also be thrown. Invocations of 'fail' in the 'Poke'
 -- monad causes this exception to be thrown.
@@ -319,6 +355,18 @@ alignBufferSize :: Int
 alignBufferSize = 32
 #endif
 
+-- | If store-core is built with the @force-alignment@ flag, then this
+-- will be a 'Just' value indicating the amount of memory that is
+-- expected in the alignment buffer used by 'PeekState' and 'PokeState'.
+-- Currently this will either be @Just 32@ or @Nothing@.
+maybeAlignmentBufferSize :: Maybe Int
+maybeAlignmentBufferSize =
+#if ALIGNED_MEMORY
+  Just alignBufferSize
+#else
+  Nothing
+#endif
+
 -- | Checks if the offset matches the expected length, and throw a
 -- 'PokeException' otherwise.
 checkOffset :: Int -> Int -> IO ()
